package function.builtin.cons;

import static sexpression.Nil.NIL;

import function.ArgumentValidator;
import function.FunctionNames;
import function.LispFunction;
import sexpression.Cons;

@FunctionNames({ "APPEND" })
public class APPEND extends LispFunction {

    public static Cons append(Cons firstList, Cons secondList) {
        Cons appendedLists = copy(firstList);

        if (firstList.isNull())
            return copy(secondList);

        getLastItem(appendedLists).setRest(copy(secondList));

        return appendedLists;
    }

    private static Cons copy(Cons list) {
        if (list.isNull())
            return NIL;

        Cons newList = new Cons(list.getFirst(), NIL);
        Cons builder = newList;

        for (Cons iterator = (Cons) list.getRest(); iterator.isCons(); iterator = (Cons) iterator.getRest()) {
            builder.setRest(new Cons(iterator.getFirst(), NIL));
            builder = (Cons) builder.getRest();
        }

        return newList;
    }

    private static Cons getLastItem(Cons list) {
        Cons tail = list;

        while (tail.getRest().isCons())
            tail = (Cons) tail.getRest();

        return tail;
    }

    private ArgumentValidator argumentValidator;

    public APPEND(String name) {
        this.argumentValidator = new ArgumentValidator(name);
        this.argumentValidator.setExactNumberOfArguments(2);
        this.argumentValidator.setEveryArgumentExpectedType(Cons.class);
    }

    @Override
    public Cons call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        Cons rest = (Cons) argumentList.getRest();
        Cons firstList = (Cons) argumentList.getFirst();
        Cons secondList = (Cons) rest.getFirst();

        return append(firstList, secondList);
    }

}
